/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Quick Lounge Applet
 *
 *  Copyright (C) 2001 The Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/gnome-ditem-edit.h>
#include <libgnomeui/gnome-window-icon.h>
#include <libgnomevfs/gnome-vfs-ops.h>
#include <glade/glade.h>

#include "file-utils.h"
#include "gtk-utils.h"
#include "quick-lounge.h"
#include "quick-box.h"
#include "quick-button.h"
#include "quick-separator.h"
#include "dlg-properties.h"
#include "dlg-pick-applications.h"


#define GLADE_FILE             "quick-lounge.glade"
#define SEPARATOR              _("Space")
#define ICON_SIZE              24
#define UPDATE_MAX_SIZE_DELAY  100


enum {
	COLUMN_DATA,
	COLUMN_ICON,
	COLUMN_NAME,
	NUM_COLUMNS
};


static GtkTargetEntry target_table[] = {
	{ "text/uri-list", 0, 0 }
};
static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);


typedef struct {
	QuickLounge   *quick_lounge;
	QuickBox      *quick_box;
	GtkWidget     *select_button;

	GladeXML      *gui;

	GtkWidget     *dialog;
	GtkWidget     *tree_view;
	GtkWidget     *btn_up;
	GtkWidget     *btn_down;
	GtkWidget     *btn_top;
	GtkWidget     *btn_bottom;

	GtkWidget     *btn_edit;
	GtkWidget     *btn_add_launcher;
	GtkWidget     *btn_add_from_menu;
	GtkWidget     *btn_add_space;
	GtkWidget     *btn_delete;

	GtkWidget     *p_minsize_spinbutton;
	GtkWidget     *p_maxsize_spinbutton;
	GtkWidget     *p_icon_size_checkbutton;

	int            n;
	GtkTreeModel  *model;

	int            timeout;
} DialogData;


/* called when the main dialog is closed. */
static void
destroy_cb (GtkWidget  *widget, 
	    DialogData *data)
{
	g_object_unref (data->gui);
	g_free (data);
}


/* called when the "close" button is pressed. */
static void
close_cb (GtkWidget  *widget, 
	  DialogData *data)
{
	gtk_widget_destroy (data->dialog);
}


/* called when the "help" button is pressed. */
static void
help_cb (GtkWidget  *widget, 
	 DialogData *data)
{
	GError *err;

	err = NULL;  
	gnome_help_display ("quick-lounge-applet", "properties", &err); 
	
	if (err != NULL) {
		GtkWidget *dialog;
		
		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
						 0,
						 GTK_MESSAGE_ERROR,
						 GTK_BUTTONS_CLOSE,
						 _("Could not display help: %s"),
						 err->message);
		gnome_window_icon_set_from_file (GTK_WINDOW (dialog), GNOME_ICONDIR "quick-lounge-applet.png");
		gtk_window_set_screen (GTK_WINDOW (dialog), gtk_widget_get_screen (GTK_WIDGET (data->quick_lounge->applet)));
		
		g_signal_connect (G_OBJECT (dialog), "response",
				  G_CALLBACK (gtk_widget_destroy),
				  NULL);
		
		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
		
		gtk_widget_show (dialog);
		
		g_error_free (err);
	}
}


static void
select_and_view_iter (DialogData  *data,
		      GtkTreeIter *iter)
{
	GtkTreeSelection *selection;
	GtkTreePath      *tpath;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
                return;

	gtk_tree_selection_select_iter (selection, iter);
	tpath = gtk_tree_model_get_path (data->model, iter);
	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (data->tree_view),
				      tpath, NULL, TRUE, 0.5, 0.0);
	gtk_tree_path_free (tpath);
}


static int
get_iter_position (DialogData  *data,
		   GtkTreeIter *iter)
{
	GtkTreePath *tpath;
	int          position;

	tpath = gtk_tree_model_get_path (data->model, iter);
	position = gtk_tree_path_get_indices (tpath)[0];
	gtk_tree_path_free (tpath);

	return position;
}


static GtkWidget *
tree_view_move_iter_to (DialogData  *data, 
			GtkTreeIter *iter,
			int          position,
			gboolean     after)
{
	GtkWidget  *button;
	GdkPixbuf  *pixbuf;
	char       *text;

	gtk_tree_model_get (data->model,
			    iter,
			    COLUMN_DATA, &button,
			    COLUMN_ICON, &pixbuf,
			    COLUMN_NAME, &text,
			    -1);
	gtk_list_store_remove (GTK_LIST_STORE (data->model), iter);

	if (after)
		gtk_list_store_append (GTK_LIST_STORE (data->model),
				       iter);
	else
		gtk_list_store_insert (GTK_LIST_STORE (data->model),
				       iter,
				       position);
	gtk_list_store_set (GTK_LIST_STORE (data->model), iter,
			    COLUMN_DATA, button,
			    COLUMN_ICON, pixbuf,
			    COLUMN_NAME, text,
			    -1);

	if (pixbuf != NULL)
		g_object_unref (pixbuf);
	g_free (text);

	return button;
}


static void
move_up_cb (GtkWidget  *widget, 
	    DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	GtkWidget        *button;
	int               position;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
                return;
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		return;

	position = get_iter_position (data, &iter) - 1;
	button = tree_view_move_iter_to (data, &iter, position, FALSE);
	select_and_view_iter (data, &iter);
	quick_box_reorder_child (data->quick_box, button, position);

	quick_lounge_save_order (data->quick_lounge);
}


static void
move_down_cb (GtkWidget  *widget, 
	      DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	GtkWidget        *button;
	int               position;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
                return;
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		return;

	position = get_iter_position (data, &iter) + 1;
	button = tree_view_move_iter_to (data, &iter, position, FALSE);
	select_and_view_iter (data, &iter);
	quick_box_reorder_child (data->quick_box, button, position);

	quick_lounge_save_order (data->quick_lounge);
}


static void
move_bottom_cb (GtkWidget  *widget, 
		DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	GtkWidget        *button;
	int               position;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
                return;
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		return;

	position = get_iter_position (data, &iter);
	button = tree_view_move_iter_to (data, &iter, data->n - 2, TRUE);
	select_and_view_iter (data, &iter);
	quick_box_reorder_child (data->quick_box, button, data->n - 1);

	quick_lounge_save_order (data->quick_lounge);
}


static void
move_top_cb (GtkWidget  *widget, 
	     DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	GtkWidget        *button;
	int               position;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
                return;
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		return;

	position = get_iter_position (data, &iter);
	button = tree_view_move_iter_to (data, &iter, 0, FALSE);
	select_and_view_iter (data, &iter);
	quick_box_reorder_child (data->quick_box, button, 0);

	quick_lounge_save_order (data->quick_lounge);
}


static void
update_sensitivity (DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter, first_iter, last_iter;
	gboolean          first_selected, last_selected;
	GtkWidget        *button;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
                return;
	
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter)
	    || (data->n == 0)) {
		gtk_widget_set_sensitive (data->btn_up, FALSE);
		gtk_widget_set_sensitive (data->btn_down, FALSE);
		gtk_widget_set_sensitive (data->btn_top, FALSE);
		gtk_widget_set_sensitive (data->btn_bottom, FALSE);
		gtk_widget_set_sensitive (data->btn_edit, FALSE);
		gtk_widget_set_sensitive (data->btn_delete, FALSE);

		return;
	}

	gtk_widget_set_sensitive (data->btn_edit, TRUE);
	gtk_widget_set_sensitive (data->btn_delete, TRUE);
	
	gtk_tree_model_get_iter_first (data->model, &first_iter);
	gtk_tree_model_iter_nth_child (data->model, &last_iter, NULL, data->n - 1);

	first_selected = gtk_tree_selection_iter_is_selected (selection, &first_iter);
	last_selected = gtk_tree_selection_iter_is_selected (selection, &last_iter);

	gtk_widget_set_sensitive (data->btn_up, ! first_selected);
	gtk_widget_set_sensitive (data->btn_top, ! first_selected);
	gtk_widget_set_sensitive (data->btn_down, ! last_selected);
	gtk_widget_set_sensitive (data->btn_bottom, ! last_selected);

	gtk_tree_model_get (data->model,
			    &iter,
			    COLUMN_DATA, &button,
			    -1);
	gtk_widget_set_sensitive (data->btn_edit, 
				  ! QUICK_IS_SEPARATOR (button));
}


static void
selection_changed_cb (GtkTreeSelection *selection,
		      gpointer          user_data)
{
	DialogData *data = user_data;
	update_sensitivity (data);
}


static void
update_list (DialogData *data)
{
	GList        *scan;
	GtkBox       *box = GTK_BOX (data->quick_box);
	GtkTreeIter   select_iter;

	gtk_list_store_clear (GTK_LIST_STORE (data->model));

	data->n = g_list_length (box->children);
	for (scan = g_list_last (box->children); scan; scan = scan->prev) {
		GtkBoxChild *child_info = scan->data;
		QuickButton *button;
		GdkPixbuf   *pixbuf;
		GtkTreeIter  iter;
		double       scale;
		char        *utf8_name;
		int          w, h;

		if (! QUICK_IS_BUTTON (child_info->widget))
			continue;
		button = QUICK_BUTTON (child_info->widget);

		if (QUICK_IS_SEPARATOR (button)) { /* Space */
			utf8_name = g_locale_to_utf8 (SEPARATOR, -1, NULL, NULL, NULL);
			gtk_list_store_prepend (GTK_LIST_STORE (data->model), 
						&iter);

			if (child_info->widget == data->select_button) 
				select_iter = iter;

			gtk_list_store_set (GTK_LIST_STORE (data->model), 
					    &iter,
					    COLUMN_DATA, button,
					    COLUMN_NAME, utf8_name,
					    -1);
			g_free (utf8_name);
			continue;
		}

		/* Application */

		/* icon */
		
		w = gdk_pixbuf_get_width (button->pixbuf);
		h = gdk_pixbuf_get_height (button->pixbuf);
		scale = MIN ((double) ICON_SIZE / w, (double) ICON_SIZE / h);
		w = MAX ((double) w * scale, 1);
		h = MAX ((double) h * scale, 1);

		pixbuf = gdk_pixbuf_scale_simple (button->pixbuf, w, h,
						  GDK_INTERP_BILINEAR);

		/**/

		gtk_list_store_prepend (GTK_LIST_STORE (data->model), &iter);

		if (child_info->widget == data->select_button) 
			select_iter = iter;

		gtk_list_store_set (GTK_LIST_STORE (data->model), &iter,
				    COLUMN_DATA, button,
				    COLUMN_ICON, pixbuf,
				    COLUMN_NAME, button->text,
				    -1);
		g_object_unref (pixbuf);
	}

	if (data->select_button != NULL) {
		select_and_view_iter (data, &select_iter);
		data->select_button = NULL;
	}
}


static void
add_columns (GtkTreeView *treeview)
{
	GtkCellRenderer   *renderer;
	GtkTreeViewColumn *column;

	/* The Name column. */

	column = gtk_tree_view_column_new ();

	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_set_attributes (column, renderer,
                                             "pixbuf", COLUMN_ICON,
                                             NULL);

	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_set_attributes (column, renderer,
                                             "text", COLUMN_NAME,
                                             NULL);

	/* gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE); FIXME */
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
}


static void edit_cb (GtkWidget *widget, DialogData *data);


static void
add_launcher_cb (GtkWidget  *widget, 
		 DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	int               pos = -1;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
		return;
	
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		pos = -2;
	else
		pos = get_iter_position (data, &iter);

	quick_lounge_new_launcher (data->quick_lounge, pos);
}


static void
add_from_menu_cb (GtkWidget  *widget, 
		  DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	int               pos = -1;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
		return;
	
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		pos = data->n - 1;
	else
		pos = get_iter_position (data, &iter);

	dlg_pick_applications (data->quick_lounge, pos);
}


static void
add_space_cb (GtkWidget  *widget, 
	      DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	int               pos = -1;
	gboolean          empty = FALSE;
	GtkWidget        *button;
	char             *utf8_name;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
		return;
	
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		empty = TRUE;
	else
		pos = get_iter_position (data, &iter);

	/* add the separator */
	
	if (empty)
		button = quick_box_add_separator (data->quick_box, -1);
	else
		button = quick_box_add_separator (data->quick_box, pos + 1);
	
	/* update the tree view */
	
	utf8_name = g_locale_to_utf8 (SEPARATOR, -1, NULL, NULL, NULL);
	if (empty)
		gtk_list_store_append (GTK_LIST_STORE (data->model), 
				       &iter);
	else
		gtk_list_store_insert_after (GTK_LIST_STORE (data->model), &iter, &iter);
	
	gtk_list_store_set (GTK_LIST_STORE (data->model), 
			    &iter,
			    COLUMN_DATA, button,
			    COLUMN_NAME, utf8_name,
			    -1);
	g_free (utf8_name);

	data->n++;
	
	/* Save changes */
	
	quick_lounge_save_order (data->quick_lounge);
	
	/* select and view */
	
	select_and_view_iter (data, &iter);
}


static void
delete_cb (GtkWidget  *widget, 
	   DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	GtkWidget        *button;
	int               pos;
	char             *uri;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
		return;
	
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		return;
	else
		pos = get_iter_position (data, &iter);

	gtk_tree_model_get (data->model,
			    &iter,
			    COLUMN_DATA, &button,
			    -1);

	uri = g_object_get_data (G_OBJECT (button), "uri");
	if (uri != NULL)
		gnome_vfs_unlink (uri);

	gtk_container_remove (GTK_CONTAINER (data->quick_box), button);
	gtk_list_store_remove (GTK_LIST_STORE (data->model), &iter);
	quick_lounge_save_order (data->quick_lounge);

	data->n--;
}


static void
edit_cb (GtkWidget  *widget, 
	 DialogData *data)
{
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	GtkWidget        *button;
	GnomeDesktopItem *ditem;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	if (selection == NULL)
		return;
	
	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
		return;

	gtk_tree_model_get (data->model,
			    &iter,
			    COLUMN_DATA, &button,
			    -1);

	ditem = g_object_get_data (G_OBJECT (button), "ditem");
	if (ditem == NULL)
		return;

	quick_lounge_util__item_properties (data->quick_lounge, button);
}


static void
min_size_value_changed_cb (GtkWidget  *widget,
			   DialogData *data)
{
	quick_lounge_set_min_visible_cols (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_minsize_spinbutton)));
}


static gboolean
change_max_size (gpointer user_data)
{
	DialogData *data = user_data;

	g_source_remove (data->timeout);
	data->timeout = 0;

	quick_lounge_set_max_size (data->quick_lounge, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton)));

	return FALSE;
}


static void
max_size_value_changed_cb (GtkWidget  *widget,
			   DialogData *data)
{
	if (data->timeout != 0)
		g_source_remove (data->timeout);
	data->timeout = g_timeout_add (UPDATE_MAX_SIZE_DELAY, change_max_size, data);
}


static void
icon_size_toggled_cb (GtkWidget  *widget,
		      DialogData *data)
{
	quick_lounge_set_icon_size_follows_panel (data->quick_lounge, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->p_icon_size_checkbutton)));
}


static GtkWidget *
get_button_from_uri (QuickBox   *quick_box,
		     const char *uri1)
{
	GtkBox *box = GTK_BOX (quick_box);
	GList  *scan;

	for (scan = box->children; scan; scan = scan->next) {
		GtkBoxChild *child = scan->data;
		const char  *uri2;

		uri2 = g_object_get_data (G_OBJECT (child->widget), "uri");
		if (uri2 == NULL)
			continue;
		if (strcmp (uri1, uri2) == 0)
			return child->widget;
	}

	return NULL;
}


static int
get_child_position (QuickBox  *quick_box,
		    GtkWidget *widget)
{
	GList  *scan;
	int     pos = 0;

	for (scan = GTK_BOX (quick_box)->children; scan; scan = scan->next) {
		GtkBoxChild *child = scan->data;

		if (child->widget == widget)
			return pos;
		pos++;
	}
	return -1;
}


static void  
drag_data_received  (GtkWidget          *widget,
		     GdkDragContext     *context,
		     int                 x,
		     int                 y,
		     GtkSelectionData   *data,
		     guint               info,
		     guint               time,
		     gpointer            extra_data)
{
	DialogData   *ddata = extra_data;
	QuickLounge  *quick_lounge = ddata->quick_lounge;
	QuickBox     *quick_box = ddata->quick_box;
        GList        *list, *scan;
	int           pos;
	GtkTreePath  *pos_path;
	GtkTreeViewDropPosition drop_pos;

        if (! ((data->length >= 0) && (data->format == 8))) {
                gtk_drag_finish (context, FALSE, FALSE, time);
                return;
        }

	if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (ddata->tree_view),
					       x, y,
					       &pos_path,
					       &drop_pos)) {
		pos = gtk_tree_path_get_indices (pos_path)[0];
		gtk_tree_path_free (pos_path);
	} else
		pos = 0;

	list = get_file_list_from_url_list ((char *) data->data);
	for (scan = list; scan; scan = scan->next) {
		char      *src_uri = scan->data;
		char      *uri;
		GtkWidget *button;

		if (! is_desktop_file (src_uri))
			continue;

		uri = g_strconcat (quick_lounge->location,
				   "/",
				   file_name_from_path (src_uri),
				   NULL);

		if (strcmp (src_uri, uri) == 0) {
			button = get_button_from_uri (quick_box, uri);

			if (button != NULL) {
				int button_pos;
				button_pos = get_child_position (quick_box, button);
				if (button_pos != -1) 
					quick_box_reorder_child (quick_box, button, pos);
			}

			g_free (uri);
			continue;
		}

		g_free (uri);
		uri = quick_lounge_util__get_unique_uri (quick_lounge);

		if (! file_copy (src_uri, uri)) {
			g_free (uri);
			continue;
		}

		if ((drop_pos == GTK_TREE_VIEW_DROP_BEFORE) ||
		    (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE))
			pos--;

		quick_box_add_button (quick_box, uri, ++pos);
	}
	path_list_free (list);

	gtk_drag_finish (context, TRUE, FALSE, time);
	quick_lounge_save_order (quick_lounge);
	dlg_properties_update (quick_lounge->prop_dialog);
}


static gboolean
drag_motion (GtkWidget          *widget,
	     GdkDragContext     *context,
	     gint                x,
	     gint                y,
	     guint               time,
	     gpointer            extra_data)
{
	DialogData              *ddata = extra_data;
	GtkTreePath             *pos_path;
	GtkTreeViewDropPosition  drop_pos;

	if (! gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (ddata->tree_view),
						 x, y,
						 &pos_path,
						 &drop_pos)) 
		pos_path = gtk_tree_path_new_first ();
	
	gtk_tree_view_set_drag_dest_row  (GTK_TREE_VIEW (ddata->tree_view),
					  pos_path,
					  drop_pos);
	gtk_tree_path_free (pos_path);

	return TRUE;
}


static void  
drag_data_get  (GtkWidget        *widget,
		GdkDragContext   *context,
		GtkSelectionData *selection_data,
		guint             info,
		guint             time,
		gpointer          data)
{
	DialogData       *ddata = data;
        char             *target;
	const char       *uri;
	GtkTreeIter       iter;
	GtkTreeSelection *selection;
	GtkWidget        *button;

        target = gdk_atom_name (selection_data->target);
        if (strcmp (target, "text/uri-list") != 0) {
		g_free (target);
		return;
	}
        g_free (target);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ddata->tree_view));
	gtk_tree_selection_get_selected (selection, NULL, &iter);
	gtk_tree_model_get (ddata->model,
			    &iter,
			    COLUMN_DATA, &button,
			    -1);
	uri = g_object_get_data (G_OBJECT (button), "uri");
	if (uri == NULL) 
                return;

        gtk_selection_data_set (selection_data, 
                                selection_data->target,
                                8, 
				uri,
                                strlen (uri));
}





GtkWidget *
dlg_properties (QuickLounge  *quick_lounge,
		GtkWidget    *select_button)
{
	DialogData       *data;
	GtkWidget        *btn_close;
	GtkWidget        *btn_help;
	GtkTreeSelection *selection;

	quick_lounge_set_textdomain ();

	data = g_new0 (DialogData, 1);

	data->quick_lounge = quick_lounge;
	data->quick_box = QUICK_BOX (quick_lounge->quick_box);
	data->select_button = select_button;
	data->timeout = 0;

	data->gui = glade_xml_new (GLADEDIR "/" GLADE_FILE , NULL, NULL);
        if (! data->gui) {
		g_free (data);
                g_warning ("Could not find " GLADE_FILE "\n");
                return NULL;
        }

	/* Get the widgets. */

	data->dialog = glade_xml_get_widget (data->gui, "properties_dialog");
	data->tree_view = glade_xml_get_widget (data->gui, "p_apps_treeview");
	data->btn_up = glade_xml_get_widget (data->gui, "p_up_button");
	data->btn_down = glade_xml_get_widget (data->gui, "p_down_button");
	data->btn_top = glade_xml_get_widget (data->gui, "p_top_button");
	data->btn_bottom = glade_xml_get_widget (data->gui, "p_bottom_button");

	data->btn_edit = glade_xml_get_widget (data->gui, "p_edit_button");
	data->btn_add_launcher = glade_xml_get_widget (data->gui, "p_add_launcher_button");
	data->btn_add_from_menu = glade_xml_get_widget (data->gui, "p_add_from_menu_button");
	data->btn_add_space = glade_xml_get_widget (data->gui, "p_add_space_button");
	data->btn_delete = glade_xml_get_widget (data->gui, "p_delete_button");

	data->p_minsize_spinbutton = glade_xml_get_widget (data->gui, "p_minsize_spinbutton");
	data->p_maxsize_spinbutton = glade_xml_get_widget (data->gui, "p_maxsize_spinbutton");
	data->p_icon_size_checkbutton = glade_xml_get_widget (data->gui, "p_icon_size_checkbutton");

	btn_close = glade_xml_get_widget (data->gui, "p_close_button");
	btn_help = glade_xml_get_widget (data->gui, "p_help_button");

	/* Set widgets data. */

	data->model = GTK_TREE_MODEL (gtk_list_store_new (NUM_COLUMNS, 
							  G_TYPE_POINTER,
							  GDK_TYPE_PIXBUF,
							  G_TYPE_STRING));
	gtk_tree_view_set_model (GTK_TREE_VIEW (data->tree_view), data->model);
	
	g_object_unref (data->model);
	add_columns (GTK_TREE_VIEW (data->tree_view));

	/**/

	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_minsize_spinbutton), quick_box_get_min_visible_cols (data->quick_box));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_maxsize_spinbutton), quick_box_get_max_visible_cols (data->quick_box));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->p_icon_size_checkbutton), quick_box_get_icon_size_follows_panel (data->quick_box));

	/* Set the signals handlers. */
	
	g_signal_connect (G_OBJECT (data->dialog), 
			  "destroy",
			  G_CALLBACK (destroy_cb),
			  data);
	g_signal_connect (G_OBJECT (btn_close), 
			  "clicked",
			  G_CALLBACK (close_cb),
			  data);
	g_signal_connect (G_OBJECT (btn_help), 
			  "clicked",
			  G_CALLBACK (help_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_up), 
			  "clicked",
			  G_CALLBACK (move_up_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_down), 
			  "clicked",
			  G_CALLBACK (move_down_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_top), 
			  "clicked",
			  G_CALLBACK (move_top_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_bottom), 
			  "clicked",
			  G_CALLBACK (move_bottom_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_add_launcher), 
			  "clicked",
			  G_CALLBACK (add_launcher_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_add_from_menu), 
			  "clicked",
			  G_CALLBACK (add_from_menu_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_add_space), 
			  "clicked",
			  G_CALLBACK (add_space_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_delete), 
			  "clicked",
			  G_CALLBACK (delete_cb),
			  data);
	g_signal_connect (G_OBJECT (data->btn_edit), 
			  "clicked",
			  G_CALLBACK (edit_cb),
			  data);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
	g_signal_connect (selection,
                          "changed",
                          G_CALLBACK (selection_changed_cb),
			  data);

	g_signal_connect (G_OBJECT (data->p_minsize_spinbutton),
                          "value_changed",
                          G_CALLBACK (min_size_value_changed_cb),
			  data);
	g_signal_connect (G_OBJECT (data->p_maxsize_spinbutton),
                          "value_changed",
                          G_CALLBACK (max_size_value_changed_cb),
			  data);
	g_signal_connect (G_OBJECT (data->p_icon_size_checkbutton),
			  "toggled",
                          G_CALLBACK (icon_size_toggled_cb),
			  data);

	/* Drag & Drop */

	gtk_drag_dest_set (GTK_WIDGET (data->tree_view),
                           GTK_DEST_DEFAULT_ALL,
                           target_table, n_targets,
			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
	
	g_signal_connect (G_OBJECT (data->tree_view), 
			  "drag_data_received",
			  G_CALLBACK (drag_data_received), 
			  data);
	g_signal_connect (G_OBJECT (data->tree_view), 
			  "drag_motion",
			  G_CALLBACK (drag_motion), 
			  data);
	
	gtk_drag_source_set (data->tree_view,
			     GDK_BUTTON1_MASK,
			     target_table, n_targets, 
			     GDK_ACTION_COPY);
	g_signal_connect (G_OBJECT (data->tree_view),
			  "drag_data_get",
			  G_CALLBACK (drag_data_get), 
			  data);

	/* run dialog. */

	g_object_set_data (G_OBJECT (data->dialog), "dialog_data", data);

	gtk_widget_show (data->dialog);
	
	update_list (data);
	update_sensitivity (data);

	return data->dialog;
}


void
dlg_properties_update (GtkWidget *dialog)
{
	DialogData *data;

	if (dialog == NULL)
		return;

	data = g_object_get_data (G_OBJECT (dialog), "dialog_data");
	if (data == NULL)
		return;

	update_list (data);
}


void
dlg_properties_select_button (GtkWidget *dialog, 
			      GtkWidget *button)
{
	DialogData *data;

	if (dialog == NULL)
		return;

	data = g_object_get_data (G_OBJECT (dialog), "dialog_data");
	if (data == NULL)
		return;

	data->select_button = button;
	update_list (data);
}
